@@ -15,13 +15,12 @@ use XML::LibXML;
 use Bio::CIPRES::Job;
 use Bio::CIPRES::Error;
 
-our $VERSION = 0.002;
-our $UA      = 'Bio::CIPRES';
+our $VERSION = '0.003';
 our $SERVER  = 'cipresrest.sdsc.edu';
 our $API     = 'cipresrest/v1';
 our $DOMAIN  = 'Cipres Authentication';
 
-my %defaults = (
+my %required = ( # must be defined after config parsing
     url     => "https://$SERVER/$API/",
     timeout => 60,
     app_id  => 'cipres_perl-E9B8D52FA2A54472BF13F25E4CD957D4',
@@ -29,6 +28,21 @@ my %defaults = (
     pass    => undef,
 );
 
+my %umb_only = ( # only for UMBRELLA auth
+    eu           => undef, # if this is defined, then the rest are:
+    eu_email     => undef, # REQUIRED
+    app_name     => undef, # REQUIRED
+    eu_instition => undef, # optional
+    eu_country   => undef, # optional
+);
+
+my @eu_headers = qw/
+    eu
+    eu_email
+    eu_institution
+    eu_country
+/;
+
 sub new {
 
     my ($class, %args) = @_;
@@ -39,13 +53,14 @@ sub new {
 
     # setup user agent
     $self->{agent} = LWP::UserAgent->new(
-        agent    => "$UA/$VERSION",
+        agent    => __PACKAGE__ . "/$VERSION",
         ssl_opts => {verify_hostname => 0},
         timeout  => $self->{cfg}->{timeout},
     );
-    $self->{agent}->default_header(
-        'cipres-appkey' => $self->{cfg}->{app_id}
-    );
+   
+    # create URI object for easier protocol/port parsing
+    $self->{uri} = URI->new( $self->{cfg}->{url} );
+
     my $netloc = join ':', $self->{uri}->host, $self->{uri}->port;
     $self->{agent}->credentials(
         $netloc,
@@ -54,6 +69,29 @@ sub new {
         $self->{cfg}->{pass}
     );
 
+    my %headers = ( 'cipres-appkey' => $self->{cfg}->{app_id} );
+
+    $self->{account} = $self->{cfg}->{user};
+
+    # UMBRELLA headers
+    if (defined $self->{cfg}->{eu}) {
+        croak "eu_email required for UMBRELLA authentication"
+            if (! defined $self->{cfg}->{'eu_email'});
+        croak "app_name required for UMBRELLA authentication"
+            if (! defined $self->{cfg}->{'app_name'});
+        for my $h (@eu_headers) {
+            my $val = $self->{cfg}->{$h} // next;
+            $h =~ s/_/\-/g;
+            $headers{"cipres-$h"} = $val;
+        }
+
+        $self->{account}
+            = uri_escape( "$self->{cfg}->{app_name}.$self->{cfg}->{eu}" );
+    }
+
+    $self->{uri}->path("/$API/job/$self->{account}");
+    $self->{agent}->default_header(%headers);
+
     return $self;
 
 }
@@ -64,7 +102,7 @@ sub _parse_args {
     my ($fn_cfg) = delete $args{conf};
 
     # set defaults
-    $self->{cfg} = {%defaults}; # copy, don't reference!
+    $self->{cfg} = {%required}; # copy, don't reference!
 
     # read from config file if asked, overwriting defaults
     if (defined $fn_cfg) {
@@ -81,12 +119,13 @@ sub _parse_args {
     $self->{cfg}->{$_} = $args{$_} for (keys %args);
 
     # check that all defined fields are valid
-    my @extra = grep {! exists $defaults{$_}} keys %{ $self->{cfg} };
+    my @extra = grep {! exists $required{$_} && ! exists $umb_only{$_}}
+        keys %{ $self->{cfg} };
     croak "Unexpected config variables found (@extra) -- check syntax"
         if (scalar @extra);
 
     # check that all required fields are defined
-    my @missing = grep {! defined $self->{cfg}->{$_}} keys %defaults;
+    my @missing = grep {! defined $self->{cfg}->{$_}} keys %required;
     croak "Required config variables missing (@missing) -- check syntax"
         if (scalar @missing);
 
@@ -94,11 +133,10 @@ sub _parse_args {
   
     # Do necessary url-encoding
     for (qw/user pass/) {
-        $self->{cfg}->{$_} =  uri_escape( $self->{cfg}->{$_} );
+        $self->{cfg}->{$_} = uri_escape( $self->{cfg}->{$_} );
     }
-   
-    # add auth info to string
-    $self->{uri} = URI->new( $self->{cfg}->{url} );
+
+    return 1;
 
 }
 
@@ -106,11 +144,9 @@ sub list_jobs {
 
     my ($self) = @_;
 
-    my $res = $self->_get(
-        "$self->{uri}/job/$self->{cfg}->{user}?expand=true"
-    );
-
+    my $res = $self->_get( "$self->{uri}?expand=true" );
     my $dom = XML::LibXML->load_xml('string' => $res);
+
     return map {
         Bio::CIPRES::Job->new( agent => $self->{agent}, dom => $_ )
     } $dom->findnodes('/joblist/jobs/jobstatus');
@@ -120,11 +156,10 @@ sub list_jobs {
 sub get_job {
 
     my ($self, $handle) = @_;
-    my $res = $self->_get(
-        "$self->{uri}/job/$self->{cfg}->{user}/$handle"
-    );
 
+    my $res = $self->_get( "$self->{uri}/$handle" );
     my $dom = XML::LibXML->load_xml('string' => $res);
+
     return Bio::CIPRES::Job->new(
         agent => $self->{agent},
         dom   => $dom,
@@ -136,12 +171,9 @@ sub submit_job {
 
     my ($self, @args) = @_;
 
-    my $res = $self->_post(
-        "$self->{uri}/job/$self->{cfg}->{user}",
-        @args,
-    );
-
+    my $res = $self->_post( $self->{uri}, @args );
     my $dom = XML::LibXML->load_xml('string' => $res);
+
     return Bio::CIPRES::Job->new(
         agent => $self->{agent},
         dom   => $dom,
@@ -245,21 +277,64 @@ user (check L<SEE ALSO> for links to tool documentation).
         conf => "$ENV{HOME}/.cipres"
     );
 
-Create a new C<Bio::CIPRES> object. There are three required parameters:
-username (C<user>), passphrase (C<pass>), and application ID (C<app_id>).
-These can either be passed in on the constructor or read in from a
-configuration file, as demonstrated above. The configuration file should
-contain key=value pairs, one pair per line, as in:
+Create a new C<Bio::CIPRES> object. There are a number of required and
+optional parameters which can be specified in the constructor or read from a
+configuration file. The configuration file should contain key=value pairs, one
+pair per line, as in:
 
     user=foo
     pass=bar
-    app_id=foo_bar_baz
+    app_id=foo_bar-12345
+
+Required parameters (no defaults):
+
+=over 1
+
+=item * user - the username of your registered CIPRES REST account
+
+=item * pass - the password of your registered CIPRES REST account
+
+=back
 
 The passphrase must be stored in plaintext, so the usual precautions apply
 (e.g. the file should not be world-readable). If possible, find another way to
 retrieve the passphrase within your code and pass it in directly as a method
 argument.
 
+Optional parameters (must be defined but defaults are provided):
+
+=over 1
+
+=item * app_id - override the application ID assigned to Bio::CIPRES 
+
+=item * url - override the default base REST url (don't change this unless you
+know what you're doing).
+
+=item * timeout - set the network timeout for HTTP requests
+
+=back
+
+UMBRELLA parameters:
+
+These parameters are only for use with UMBRELLA applications (you will need
+you register your own UMBRELLA application to use this functionality). They
+are not needed for non-UMBRELLA applications, but if C<eu> is defined then
+UMBRELLA is assumed and C<eu_email> and C<app_name> must be defined as well.
+
+=over 1
+
+=item * eu - end user name
+
+=item * eu_email - end user email address
+
+=item * app_name - UMBRELLA application name as registered with CIPRES
+
+=item * eu_institution - end user institution (currently optional)
+
+=item * eu_country - end user two-letter country code (currently optional)
+
+=back
+
 =item B<submit_job>
 
     my $job = $ua->submit_job( %params );
@@ -302,6 +377,14 @@ found.
 
 =back
 
+=head1 TESTING
+
+The distribution can be installed and tested in the usual ways. Note however,
+that running the full test suite requires CIPRES REST credentials (not shipped
+with package for obvious reasons). If a credentials file is found at
+"$ENV{HOME}/.cipres", the full test suite will be run -- otherwise only
+rudimentary tests will be run and most will be skipped.
+
 =head1 CAVEATS AND BUGS
 
 This is code is in alpha testing stage and the API is not guaranteed to be
@@ -311,6 +394,10 @@ Currently the use of UMBRELLA authentication is not implemented.
 
 Please report bugs to the author.
 
+=head1 SEE ALSO
+
+L<https://www.phylo.org/restusers/documentation.action>
+
 =head1 AUTHOR
 
 Jeremy Volkening <jdv@base2bio.com>
